
html {
  font-size: 18px;
}

#main-container {
  width: 80%;
  font-family: Sans-Serif;
  font-size: 20px;
  margin-left: auto;
  margin-right: auto
}

#tab-content-container {
  margin: 20px;
}

#data-table {
  font-size: 14px;
  line-height: 14px;
}

#data-table td {
  padding-top: 0;
  padding-bottom: 0;
}

.plot-button {
    border: 1px solid #ccc;
    background-color: #f9f9f9;
    cursor: pointer;
}

.plot-button.selected {
    background-color: #007bff;
    color: white;
    border-color: #007bff;
}

/* Job table styling */
.job-table {
    background-color: white;
    border-radius: 4px;
    overflow: hidden;
}

.job-table th {
    background-color: #f8f9fa;
    color: #495057;
    font-weight: 600;
    text-align: left;
    padding: 12px 16px;
    border-bottom: 2px solid #dee2e6;
    font-size: 13px;
    text-transform: uppercase;
    letter-spacing: 0.5px;
}

.job-table td {
    padding: 12px 16px;
    border-bottom: 1px solid #e9ecef;
    vertical-align: middle;
}

.job-table tr:last-child td {
    border-bottom: none;
}

.job-table tr:hover {
    background-color: #f8f9fa;
}

.job-table button {
    padding: 6px 12px;
    font-size: 13px;
    border-radius: 4px;
    border: 1px solid #dee2e6;
    background-color: white;
    cursor: pointer;
    transition: all 0.2s;
}

.job-table button:hover {
    background-color: #e9ecef;
    border-color: #adb5bd;
}
